#!/usr/bin/python
#
# Copyright (c) 2021 Cole Neubauer, (@coleneubauer)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
module: azure_rm_adgroup
version_added: "1.6.0"
short_description: Manage Azure Active Directory group
description:
    - Create, update or delete Azure Active Directory group.
options:
    state:
        description:
            - Assert the state of the resource group. Use C(present) to create or update and C(absent) to delete.
        default: present
        choices:
            - absent
            - present
        type: str
    object_id:
        description:
            - The object id for the ad group.
            - Can be used to reference when updating an existing group.
            - Ignored when attempting to create a group.
        type: str
    display_name:
        description:
            - The display name of the ad group.
            - Can be used with I(mail_nickname) instead of I(object_id) to reference existing group.
            - Required when creating a new ad group.
        type: str
    mail_nickname:
        description:
            - The mail nickname of the ad group.
            - Can be used with I(display_name) instead of I(object_id) to reference existing group.
            - Required when creating a new ad group.
        type: str
    present_members:
        description:
            - The azure ad objects asserted to be members of the group.
            - This list does not need to be all inclusive. Objects that are members and not on this list remain members.
        type: list
        elements: str
    absent_members:
        description:
            - The azure ad objects asserted to not be members of the group.
        type: list
        elements: str
    present_owners:
        description:
            - The azure ad objects asserted to be owners of the group.
            - This list does not need to be all inclusive. Objects that are owners and not on this list remain members.
        type: list
        elements: str
    absent_owners:
        description:
            - The azure ad objects asserted to not be owners of the group.
        type: list
        elements: str
    raw_membership:
        description:
            - By default the group_members return property is flattened and partially filtered of non-User objects before return. \
             This argument disables those transformations.
        default: false
        type: bool
    description:
        description:
            - An optional description for the group.
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
author:
    - Cole Neubauer(@coleneubauer)
'''

EXAMPLES = '''
- name: Create Group
  azure_rm_adgroup:
    display_name: "Group-Name"
    mail_nickname: "Group-Mail-Nickname"
    description: 'fortest'
    state: 'present'

- name: Delete Group using display_name and mail_nickname
  azure_rm_adgroup:
    display_name: "Group-Name"
    mail_nickname: "Group-Mail-Nickname"
    state: 'absent'

- name: Delete Group using object_id
  azure_rm_adgroup:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    state: 'absent'

- name: Ensure Users are Members of a Group using display_name and mail_nickname
  azure_rm_adgroup:
    display_name: "Group-Name"
    mail_nickname: "Group-Mail-Nickname"
    state: 'present'
    present_members:
      - "{{ ad_object_1_object_id }}"
      - "{{ ad_object_2_object_id }}"

- name: Ensure Users are Members of a Group using object_id
  azure_rm_adgroup:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    state: 'present'
    present_members:
      - "{{ ad_object_1_object_id }}"
      - "{{ ad_object_2_object_id }}"

- name: Ensure Users are Members of a Group using object_id. Specify the group_membership return should be unfiltered
  azure_rm_adgroup:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    state: 'present'
    present_members:
      - "{{ ad_object_1_object_id }}"
      - "{{ ad_object_2_object_id }}"
    raw_membership: true

- name: Ensure Users are not Members of a Group using display_name and mail_nickname
  azure_rm_adgroup:
    display_name: "Group-Name"
    mail_nickname: "Group-Mail-Nickname"
    state: 'present'
    absent_members:
      - "{{ ad_object_1_object_id }}"

- name: Ensure Users are not Members of a Group using object_id
  azure_rm_adgroup:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    state: 'present'
    absent_members:
      - "{{ ad_object_1_object_id }}"

- name: Ensure Users are Owners of a Group using display_name and mail_nickname
  azure_rm_adgroup:
    display_name: "Group-Name"
    mail_nickname: "Group-Mail-Nickname"
    state: 'present'
    present_owners:
      - "{{ ad_object_1_object_id }}"
      - "{{ ad_object_2_object_id }}"

- name: Ensure Users are Owners of a Group using object_id
  azure_rm_adgroup:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    state: 'present'
    present_owners:
      - "{{ ad_object_1_object_id }}"
      - "{{ ad_object_2_object_id }}"

- name: Ensure Users are not Owners of a Group using display_name and mail_nickname
  azure_rm_adgroup:
    display_name: "Group-Name"
    mail_nickname: "Group-Mail-Nickname"
    state: 'present'
    absent_owners:
      - "{{ ad_object_1_object_id }}"
      - "{{ ad_object_2_object_id }}"

- name: Ensure Users are not Owners of a Group using object_id
  azure_rm_adgroup:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
    state: 'present'
    absent_owners:
      - "{{ ad_object_1_object_id }}"
      - "{{ ad_object_2_object_id }}"
'''

RETURN = '''
object_id:
    description:
        - The object_id for the group.
    type: str
    returned: always
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
display_name:
    description:
        - The display name of the group.
    returned: always
    type: str
    sample: GroupName
mail_nickname:
    description:
        - The mail alias for the group.
    returned: always
    type: str
    sample: groupname
mail_enabled:
    description:
        - Whether the group is mail-enabled. Must be false. This is because only pure security groups can be created using the Graph API.
    returned: always
    type: bool
    sample: False
security_enabled:
    description:
        - Whether the group is security-enable.
    returned: always
    type: bool
    sample: False
mail:
    description:
        - The primary email address of the group.
    returned: always
    type: str
    sample: group@contoso.com
group_owners:
    description:
        - The owners of the group.
    returned: always
    type: list
group_members:
    description:
        - The members of the group. If raw_membership is false, this contains the transitive members property. Otherwise, it contains the members property.
    returned: always
    type: list
description:
    description:
        - An optional description for the group.
    type: str
    returned: always
    sample: 'fortest'
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBase

try:
    import asyncio
    from msgraph.generated.groups.groups_request_builder import GroupsRequestBuilder
    from msgraph.generated.models.group import Group
    from msgraph.generated.groups.item.transitive_members.transitive_members_request_builder import \
        TransitiveMembersRequestBuilder
    from msgraph.generated.groups.item.group_item_request_builder import GroupItemRequestBuilder
    from msgraph.generated.models.reference_create import ReferenceCreate
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMADGroup(AzureRMModuleBase):
    def __init__(self):

        self.module_arg_spec = dict(
            object_id=dict(type='str'),
            display_name=dict(type='str'),
            mail_nickname=dict(type='str'),
            present_members=dict(type='list', elements='str'),
            present_owners=dict(type='list', elements='str'),
            absent_members=dict(type='list', elements='str'),
            absent_owners=dict(type='list', elements='str'),
            raw_membership=dict(type='bool', default=False),
            description=dict(type='str'),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
        )

        self.display_name = None
        self.mail_nickname = None
        self.object_id = None
        self.present_members = []
        self.present_owners = []
        self.absent_members = []
        self.absent_owners = []
        self.state = None
        self.results = dict(changed=False)
        self._client = None
        self.raw_membership = False

        super(AzureRMADGroup, self).__init__(derived_arg_spec=self.module_arg_spec,
                                             supports_check_mode=False,
                                             supports_tags=False,
                                             is_ad_resource=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        try:
            self._client = self.get_msgraph_client()
            ad_groups = []

            if self.display_name and self.mail_nickname:
                filter = "displayName eq '{0}' and mailNickname eq '{1}'".format(self.display_name, self.mail_nickname)
                ad_groups = asyncio.get_event_loop().run_until_complete(self.get_group_list(filter))
                if ad_groups:
                    self.object_id = ad_groups[0].id

            if self.object_id:
                ad_groups = [asyncio.get_event_loop().run_until_complete(self.get_group(self.object_id))]

            if ad_groups:
                if self.state == "present":
                    self.results["changed"] = False

                    if self.description is not None and self.description != ad_groups[0].description:
                        self.results["changed"] = True
                    else:
                        self.description = ad_groups[0].description
                    if self.display_name is not None and self.display_name != ad_groups[0].display_name:
                        self.results["changed"] = True
                    else:
                        self.display_name = ad_groups[0].display_name
                    if self.mail_nickname is not None and self.mail_nickname != ad_groups[0].mail_nickname:
                        self.results["changed"] = True
                    else:
                        self.mail_nickname = ad_groups[0].mail_nickname
                    if self.results["changed"]:
                        group = Group(
                            mail_enabled=False,
                            security_enabled=True,
                            group_types=[],
                            display_name=self.display_name,
                            mail_nickname=self.mail_nickname,
                            description=self.description
                        )

                        asyncio.get_event_loop().run_until_complete(self.update_group(ad_groups[0].id, group))
                        ad_groups = [asyncio.get_event_loop().run_until_complete(self.get_group(self.object_id))]

                elif self.state == "absent":
                    asyncio.get_event_loop().run_until_complete(self.delete_group(self.object_id))
                    ad_groups = []
                    self.results["changed"] = True
            else:
                if self.state == "present":
                    if self.display_name and self.mail_nickname:
                        group = Group(
                            mail_enabled=False,
                            security_enabled=True,
                            group_types=[],
                            display_name=self.display_name,
                            mail_nickname=self.mail_nickname,
                            description=self.description
                        )

                        ad_groups = [asyncio.get_event_loop().run_until_complete(self.create_group(group))]
                        self.results["changed"] = True
                    else:
                        raise ValueError(
                            'The group does not exist. Both display_name : {0} and mail_nickname : {1} must be passed to create a new group'
                            .format(self.display_name, self.mail_nickname))
                elif self.state == "absent":
                    self.results["changed"] = False

            if ad_groups and ad_groups[0] is not None:
                self.update_members(ad_groups[0].id)
                self.update_owners(ad_groups[0].id)
                self.results.update(self.set_results(ad_groups[0]))

        except Exception as e:
            self.fail(e)
        return self.results

    def update_members(self, group_id):
        current_members = []

        if self.present_members or self.absent_members:
            ret = asyncio.get_event_loop().run_until_complete(self.get_group_members(group_id))
            current_members = [object.id for object in ret]

        if self.present_members:
            present_members_by_object_id = self.dictionary_from_object_urls(self.present_members)

            members_to_add = list(set(present_members_by_object_id.keys()) - set(current_members))

            if members_to_add:
                for member_object_id in members_to_add:
                    asyncio.get_event_loop().run_until_complete(
                        self.add_group_member(group_id, present_members_by_object_id[member_object_id]))
                self.results["changed"] = True

        if self.absent_members:
            members_to_remove = list(set(self.absent_members).intersection(set(current_members)))

            if members_to_remove:
                for member in members_to_remove:
                    asyncio.get_event_loop().run_until_complete(self.delete_group_member(group_id, member))
                self.results["changed"] = True

    def update_owners(self, group_id):
        current_owners = []

        if self.present_owners or self.absent_owners:
            ret = asyncio.get_event_loop().run_until_complete(self.get_group_owners(group_id))
            current_owners = [object.id for object in ret.value]

        if self.present_owners:

            present_owners_by_object_id = self.dictionary_from_object_urls(self.present_owners)
            owners_to_add = list(set(present_owners_by_object_id.keys()) - set(current_owners))

            if owners_to_add:
                for owner_object_id in owners_to_add:
                    asyncio.get_event_loop().run_until_complete(
                        self.add_group_owner(group_id, present_owners_by_object_id[owner_object_id]))
                self.results["changed"] = True

        if self.absent_owners:
            owners_to_remove = list(set(self.absent_owners).intersection(set(current_owners)))

            if owners_to_remove:
                for owner in owners_to_remove:
                    asyncio.get_event_loop().run_until_complete(self.remove_group_owner(group_id, owner))
                self.results["changed"] = True

    def dictionary_from_object_urls(self, object_urls):
        objects_by_object_id = {}

        for urls in object_urls:
            object_id = urls.split("/")[-1]
            objects_by_object_id[object_id] = urls

        return objects_by_object_id

    def application_to_dict(self, object):
        return dict(
            app_id=object.app_id,
            object_id=object.object_id,
            display_name=object.display_name,
        )

    def serviceprincipal_to_dict(self, object):
        return dict(
            app_id=object.app_id,
            object_id=object.id,
            app_display_name=object.display_name,
            app_role_assignment_required=object.app_role_assignment_required
        )

    def group_to_dict(self, object):
        return dict(
            object_id=object.id,
            display_name=object.display_name,
            mail_nickname=object.mail_nickname,
            mail_enabled=object.mail_enabled,
            security_enabled=object.security_enabled,
            mail=object.mail,
            description=object.description
        )

    def user_to_dict(self, object):
        return dict(
            object_id=object.id,
            display_name=object.display_name,
            user_principal_name=object.user_principal_name,
            mail_nickname=object.mail_nickname,
            mail=object.mail,
            account_enabled=object.account_enabled,
            user_type=object.user_type
        )

    def result_to_dict(self, object):
        if object.odata_type == "#microsoft.graph.group":
            return self.group_to_dict(object)
        elif object.odata_type == "#microsoft.graph.user":
            return self.user_to_dict(object)
        elif object.odata_type == "#microsoft.graph.application":
            return self.application_to_dict(object)
        elif object.odata_type == "#microsoft.graph.servicePrincipal":
            return self.serviceprincipal_to_dict(object)
        else:
            return object.odata_type

    def set_results(self, object):
        results = self.group_to_dict(object)

        if results["object_id"] and (self.present_owners or self.absent_owners):
            ret = asyncio.get_event_loop().run_until_complete(self.get_group_owners(results["object_id"]))
            results["group_owners"] = [self.result_to_dict(object) for object in ret.value]

        if results["object_id"] and (self.present_members or self.absent_members):
            ret = asyncio.get_event_loop().run_until_complete(self.get_group_members(results["object_id"]))
            results["group_members"] = [self.result_to_dict(object) for object in ret]

        return results

    async def update_group(self, group_id, group):
        return await self._client.groups.by_group_id(group_id).patch(body=group)

    async def create_group(self, create_group):
        return await self._client.groups.post(body=create_group)

    async def delete_group(self, group_id):
        await self._client.groups.by_group_id(group_id).delete()

    async def get_group(self, group_id):
        return await self._client.groups.by_group_id(group_id).get()

    async def get_group_list(self, filter=None):
        if filter:
            request_configuration = GroupsRequestBuilder.GroupsRequestBuilderGetRequestConfiguration(
                query_parameters=GroupsRequestBuilder.GroupsRequestBuilderGetQueryParameters(
                    count=True,
                    filter=filter,
                ),
            )
            groups = await self._client.groups.get(request_configuration=request_configuration)
        else:
            groups = await self._client.groups.get()

        if groups and groups.value:
            return groups.value
        return []

    async def get_group_members(self, group_id, filters=None):
        if self.raw_membership:
            return await self.get_raw_group_members(group_id, filters)
        else:
            return await self.get_transitive_group_members(group_id, filters)

    async def get_transitive_group_members(self, group_id, filters=None):
        request_configuration = TransitiveMembersRequestBuilder.TransitiveMembersRequestBuilderGetRequestConfiguration(
            query_parameters=TransitiveMembersRequestBuilder.TransitiveMembersRequestBuilderGetQueryParameters(
                count=True,
            ),
        )
        if filters:
            request_configuration.query_parameters.filter = filters
        response = await self._client.groups.by_group_id(group_id).transitive_members.get(
            request_configuration=request_configuration)
        return response.value

    async def get_raw_group_members(self, group_id, filters=None):
        request_configuration = GroupItemRequestBuilder.GroupItemRequestBuilderGetRequestConfiguration(
            query_parameters=GroupItemRequestBuilder.GroupItemRequestBuilderGetQueryParameters(
                # this ensures service principals are returned
                # see https://learn.microsoft.com/en-us/graph/api/group-list-members?view=graph-rest-1.0&tabs=http
                expand=["members"]
            ),
        )
        if filters:
            request_configuration.query_parameters.filter = filters
        group = await self._client.groups.by_group_id(group_id).get(request_configuration=request_configuration)
        return group.members

    async def add_group_member(self, group_id, obj_id):
        request_body = ReferenceCreate(
            odata_id="https://graph.microsoft.com/v1.0/directoryObjects/{0}".format(obj_id),
        )
        await self._client.groups.by_group_id(group_id).members.ref.post(body=request_body)

    async def delete_group_member(self, group_id, member_id):
        await self._client.groups.by_group_id(group_id).members.by_directory_object_id(member_id).ref.delete()

    async def get_group_owners(self, group_id):
        request_configuration = GroupsRequestBuilder.GroupsRequestBuilderGetRequestConfiguration(
            query_parameters=GroupsRequestBuilder.GroupsRequestBuilderGetQueryParameters(
                count=True,
            ),
        )
        return await self._client.groups.by_group_id(group_id).owners.get(request_configuration=request_configuration)

    async def add_group_owner(self, group_id, obj_id):
        request_body = ReferenceCreate(
            odata_id="https://graph.microsoft.com/v1.0/users/{0}".format(obj_id),
        )
        await self._client.groups.by_group_id(group_id).owners.ref.post(body=request_body)

    async def remove_group_owner(self, group_id, obj_id):
        await self._client.groups.by_group_id(group_id).owners.by_directory_object_id(obj_id).ref.delete()


def main():
    AzureRMADGroup()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for iosxr_route_maps
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: iosxr_route_maps
short_description: Resource module to configure route maps.
description:
  - This module configures and manages the attributes of Route maps on Cisco IOSXR.
version_added: 10.2.0
author: Sagar Paul (@KB-perByte)
notes:
  - Tested against Cisco IOS-XR 7.2.2.
  - This module works with connection C(network_cli).
options:
  config:
    description: A list of configurations for route policy.
    type: list
    elements: dict
    suboptions:
      name:
        description: Name of the route policy.
        type: str
      global: &global
        description: A dictionary of configurations for route policy without any conditions
        type: dict
        suboptions:
          add: &add
            description: Add offset to the existing value
            type: dict
            suboptions:
              eigrp_metric:
                description: EIGRP metric attribute
                type: dict
                suboptions:
                  bandwidth:
                    description: <0-4294967295> Bandwidth in Kbits per second
                    type: int
                  delay:
                    description: <0-4294967295> Delay metric in 10 microsecond units
                    type: int
                  reliability:
                    description: <0-255> Reliability metric where 255 is 100% reliable
                    type: int
                  effective_bandwith:
                    type: int
                    description: <0-255> Effective bandwidth metric (Loading) where 255 is 100% loaded
                  max_transmission:
                    description: <0-65535> Maximum Transmission Unit metric of the path
                    type: int
              rip_metric:
                description: <0-16> RIP metric attribute
                type: int
          apply: &apply
            description: Apply a route policy
            type: list
            elements: dict
            suboptions:
              route_policy:
                type: str
                description: Apply a specific route policy
              route_policy_input:
                type: str
                description: ipv4/ ipv6 / name-string
          drop: &drop
            type: bool
            description: Reject this route with no further processing
          pass: &pass
            type: bool
            description: Pass this route for further processing
          prepend: &prepend
            description: Prepend to BGP AS-path
            type: dict
            suboptions:
              number_of_times:
                type: int
                description: number of times to prepend
              as_path:
                type: int
                description: <1-4294967295> 32-bit decimal number/ 16-bit decimal number as-path
              most_recent:
                type: bool
                description: Most recent Autonomous System Number
              own_as:
                type: bool
                description: Local Autonomous System Number
          suppress_route: &suppess
            type: bool
            description: Suppress specific routes when aggregating
          unsuppress_route: &unsuppess
            type: bool
            description: Unsuppress specific aggregated routes
          remove: &remove
            description: Remove all private-as entries
            type: dict
            suboptions:
              set:
                type: bool
                description: Remove all private-as entries (remove as-path private-as)
              entire_aspath:
                type: bool
                description: Remove private-AS from entire aspath
          set: &set
            description: Set a route attribute
            type: dict
            suboptions:
              administrative_distance:
                description: Administrative Distance of the prefix, <1-255> 8 bit decimal numbe
                type: int
              aigp_metric:
                description: AIGP metric attribute
                type: dict
                suboptions:
                  icrement:
                    type: bool
                    description: "+ Increment the attribute with specified value"
                  decrement:
                    description: "- Decrement the attribute by specified value"
                    type: bool
                  metric_number:
                    description: <0-4294967295>  32-bit decimal number
                    type: int
                  igp_cost:
                    description: Internal routing protocol cost
                    type: bool
              attribute_set:
                description: TE attribute-set name <0-4294967295> 32-bit decimal number
                type: str
              c_multicast_routing:
                description: Multicast Customer routing type
                type: dict
                suboptions:
                  bgp:
                    type: bool
                    description: BGP customer-multicast routing
                  pim:
                    type: bool
                    description: PIM customer-multicast routing
              community:
                description: BGP community attribute
                type: dict
                suboptions:
                  community_name:
                    type: str
                    description: Community set name
                  additive:
                    type: bool
                    description: Add to the existing community
              core_tree:
                description: Multicast Distribution Tree type
                type: dict
                suboptions:
                  ingress_replication:
                    type: bool
                    description: Ingress Replication core segment
                  ingress_replication_default:
                    type: bool
                    description: Ingress Replication Default MDT core
                  ingress_replication_partitioned:
                    type: bool
                    description: Ingress Replication Partitioned MDT core
                  mldp:
                    type: bool
                    description: MLDP core segment
                  mldp_default:
                    type: bool
                    description: MLDP Default MDT core
                  mldp_inband:
                    type: bool
                    description: MLDP Inband core
                  mldp_partitioned_mp2mp:
                    type: bool
                    description: MLDP Partitioned MP2MP MDT core
                  mldp_partitioned_p2mp:
                    type: bool
                    description: MLDP Partitioned P2MP MDT core
                  p2mp_te:
                    type: bool
                    description: P2MP TE core segment
                  p2mp_te_default:
                    type: bool
                    description: P2MP TE Default MDT core
                  p2mp_te_partitioned:
                    type: bool
                    description: P2MP TE Partitioned MDT core
                  pim_default:
                    type: bool
                    description: PIM Default MDT core
                  sr_p2mp:
                    type: bool
                    description: Segment-Routing P2MP core
              dampening:
                description: BGP route flap dampening parameters
                type: dict
                suboptions:
                  halflife:
                    type: int
                    description: Dampening penalty half-life, <1-45> Half-life time for penalty, default 15
                  max_suppress:
                    type: int
                    description: Maximum dampening penalty, <1-255> Maximum dampening penalty time, default 60
                  reuse:
                    type: int
                    description: Penalty before reusing suppressed route, <1-20000> Dampening reuse threshold, default 750
                  suppress:
                    type: int
                    description: Dampening penalty to start suppressing a route, <1-20000>  Suppress penalty threshold, default 2000
              downstream_core_tree:
                description: BGP I-PMSI/S-PMSI core tree type
                type: dict
                suboptions:
                  ingress_replication:
                    type: bool
                    description: Ingress Replication core segment
                  mldp:
                    type: bool
                    description: MLDP core segment
                  p2mp_te:
                    type: bool
                    description: P2MP TE core segment
                  sr_p2mp:
                    type: bool
                    description: Segment-Routing P2MP core
              eigrp_metric:
                description: EIGRP metric attribute
                type: dict
                suboptions:
                  bandwidth:
                    description: <0-4294967295> Bandwidth in Kbits per second
                    type: int
                  delay:
                    description: <0-4294967295> Delay metric in 10 microsecond units
                    type: int
                  reliability:
                    description: <0-255> Reliability metric where 255 is 100% reliable
                    type: int
                  effective_bandwith:
                    type: int
                    description: <0-255> Effective bandwidth metric (Loading) where 255 is 100% loaded
                  max_transmission:
                    description: <0-65535> Maximum Transmission Unit metric of the path
                    type: int
              fallback_vrf_lookup:
                description: fallback vrf look-up
                type: bool
              flow_tag:
                description: flow tag value for PBR BGP flow-tag, <1-63> 6 bit decimal number starting from 1
                type: int
              forward_class:
                description: Forward class (default value 0), <1-7> 3 bit decimal number starting from 1
                type: int
              ip_precedence:
                description: IP Precedence to classify packets, <1-7> 3 bit decimal number starting from 1
                type: int
              isis_metric:
                description: IS-IS metric attribute, <0-16777215> 24 bit decimal number
                type: int
              label:
                description: Set BGP label value, <0-1048575> 20 bit decimal number
                type: int
              label_index:
                description: Set Segment Routing label-index value, <0-1048575>  20 bit decimal number
                type: int
              label_mode:
                description: Set BGP label-mode value
                type: dict
                suboptions:
                  per_ce:
                    type: bool
                    description: Set the label mode to per-ce
                  per_prefix:
                    type: bool
                    description: Set the label mode to per-prefix
                  per_vrf:
                    type: bool
                    description: Set the label mode to per-vrf
              large_community:
                description: BGP large community attribute
                type: str
              level:
                description: Where to import route
                type: dict
                suboptions:
                  level_1:
                    type: bool
                    description: IS-IS level-1 routes
                  level_1_2:
                    type: bool
                    description: IS-IS level-1 and level-2 routes
                  level_2:
                    type: bool
                    description: IS-IS level-2 routes
              load_balance:
                description: Load-balance for ECMP ecmp-consistent
                type: bool
              lsm_root:
                description: Label Switched Multicast Root address
                type: str
              metric_type:
                description: Type of metric for destination routing protocol
                type: dict
                suboptions:
                  external:
                    type: bool
                    description: ISIS external metric-type
                  internal:
                    type: bool
                    description: ISIS internal metric-type
                  rib_metric_as_external:
                    type: bool
                    description: Use RIB metric and set ISIS external metric-type
                  rib_metric_as_internal:
                    type: bool
                    description: Use RIB metric and set ISIS internal metric-type
                  type_1:
                    type: bool
                    description: OSPF type-1 route
                  type_2:
                    type: bool
                    description: OSPF type-2 route
              mpls:
                description: MPLS traffic-eng attributeset name-string
                type: str
              next_hop:
                description: Next hop address specified in this route
                type: dict
                suboptions:
                  address:
                    type: str
                    description: next hop address
              origin:
                description: BGP origin code
                type: dict
                suboptions:
                  egp:
                    type: bool
                    description: ISIS external metric-type
                  igp:
                    type: bool
                    description: ISIS internal metric-type
                  rincomplete:
                    type: bool
                    description: Use RIB metric and set ISIS external metric-type
              ospf_metric:
                description: OSPF metric attribute
                type: int
              path_selection:
                description: BGP path selection
                type: dict
                suboptions:
                  all:
                    type: bool
                    description: BGP all advertise
                  backup:
                    description: BGP backup
                    type: dict
                    suboptions:
                      backup_decimal:
                        type: int
                        description: <1>, decimal number 1
                      advertise:
                        type: bool
                        description: Advertise the path
                      install:
                        type: bool
                        description: Install the path
                  best_path:
                    type: bool
                    description: BGP best path
                  group_best:
                    type: bool
                    description: BGP group-best advertise
                  multiplath:
                    type: bool
                    description: BGP multipath advertise
              path_color:
                description: BGP Path Color for RIB (path-color external-reach)
                type: bool
              qos_group:
                description: QoS Group to classify packets
                type: int
              rib_metric:
                description: RIB metric for table-policy
                type: int
              rip_metric:
                description: RIP metric attribute
                type: int
              rip_tag:
                description: RIP Route tag attribute
                type: int
              rt_set:
                description: Limit on routes with paths with an RT-set
                type: int
              s_pmsi:
                description: S-PMSI Advertisement type (star-g)
                type: bool
              spf_priority:
                description: OSPF SPF priority
                type: dict
                suboptions:
                  critical:
                    type: bool
                    description: Critical priority
                  high:
                    type: bool
                    description: High priority
                  medium:
                    type: bool
                    description: Medium priority
              static_p2mp_te:
                description: Static P2MP-TE tunnel
                type: str
              tag:
                description: Route tag attribute
                type: int
              traffic_index:
                description: Traffic-index for BGP policy accounting
                type: dict
                suboptions:
                  index_number:
                    type: int
                    description: 6 bit decimal number starting from 1 <1-63>
                  ignore:
                    type: bool
                    description: Remove any traffic-index setting
              upstream_core_tree:
                description: BGP Leaf AD core tree type
                type: dict
                suboptions:
                  ingress_replication:
                    type: bool
                    description: Ingress Replication core segment
                  mldp:
                    type: bool
                    description: MLDP core segment
                  p2mp_te:
                    type: bool
                    description: P2MP TE core segment
                  sr_p2mp:
                    type: bool
                    description: Segment-Routing P2MP core
              vpn_distinguisher:
                description: BGP VPN distinguisher (VD) attribute
                type: int
              weight:
                description: Weight attribute for route selection
                type: int
      if_section:
        description: A dictionary of configurations for route policy for the top level if condition for the policy
        type: dict
        suboptions: &ifcondition
          condition:
            type: str
            description:
              - the condition string, eg - aigp-metric eq 23 and as-path in tmp1 and
                community is-empty and community matches-any test1 (don't add then at end)
          add: *add
          apply: *apply
          drop: *drop
          pass: *pass
          prepend: *prepend
          suppress_route: *suppess
          unsuppress_route: *unsuppess
          remove: *remove
          set: *set
      elseif_section: &elifmodel
        description: A list of elif configurations that would follow along with the top level if
        type: list
        elements: dict
        suboptions: *ifcondition
      else_section:
        description: A dictionary of configurations that would be considered in the else block
        type: dict
        suboptions:
          global: *global
          if_section:
            description: A dictionary of configurations for route policy for the nested if condition, under top level else
            type: dict
            suboptions: *ifcondition
          elseif_section: *elifmodel
          else_section:
            description: A dictionary of configurations for nested else, does not support if/ elseif
            type: dict
            suboptions:
              add: *add
              apply: *apply
              drop: *drop
              pass: *pass
              prepend: *prepend
              suppress_route: *suppess
              unsuppress_route: *unsuppess
              remove: *remove
              set: *set
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be and aggregate of the output received from the IOS XR
        device by executing the command B(show running-config route-policy <policy_name>)
        for per route-policy.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and
        transform it into structured data in the format as per the resource module
        argspec and the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of
        C(running_config) option should be the aggregate of the output of
        command I(show running-config route-policy <policy_name>) that gives individual
        route-policy details and executed on device.
        For state I(parsed) active connection to remote host is not required.
    type: str
    choices:
      - merged
      - replaced
      - overridden
      - purged
      - rendered
      - gathered
      - parsed
    default: merged
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# viosxr#show running-config | include route-policy
#

- name: Merge route-policy configuration
  cisco.iosxr.iosxr_route_maps:
    state: merged
    config:
      - global:
          apply:
            - route_policy: A_NEW_ROUTE_POLICY
          set:
            community:
              additive: true
              community_name: (11011:1001)
            weight: 20000
        name: SIMPLE_GLOBAL_ROUTE_POLICY
      - else_section:
          global:
            drop: true
        if_section:
          condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
          pass: true
        name: SIMPLE_CONDITION_ROUTE_POLICY
      - else_section:
          else_section:
            drop: true
          if_section:
            condition: destination in A_RANDOM_POLICY
            pass: true
            set:
              community:
                additive: true
                community_name: (101010:1)
        if_section:
          condition: as-path in (ios-regex '_3117_', ios-regex '_600_')
          drop: true
        name: COMPLEX_ROUTE_POLICY
      - else_section:
          global:
            pass: true
        if_section:
          condition: community matches-any (9119:1001) or community matches-any (11100:1001)
          drop: true
        name: COMPLEX_CONDITION_ROUTE_POLICY

# Task Output
# -----------
#
# before:
# - {}
# commands:
# - route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# - apply A_NEW_ROUTE_POLICY
# - set community (11011:1001) additive
# - set weight 20000
# - end-policy
# - route-policy SIMPLE_CONDITION_ROUTE_POLICY
# - if destination in SIMPLE_CONDITION_ROUTE_POLICY then
# - pass
# - else
# - drop
# - endif
# - end-policy
# - route-policy COMPLEX_ROUTE_POLICY
# - if as-path in (ios-regex '_3117_', ios-regex '_600_') then
# - drop
# - else
# - if destination in A_RANDOM_POLICY then
# - pass
# - set community (101010:1) additive
# - else
# - drop
# - endif
# - endif
# - end-policy
# - route-policy COMPLEX_CONDITION_ROUTE_POLICY
# - if community matches-any (9119:1001) or community matches-any (11100:1001) then
# - drop
# - else
# - pass
# - endif
# - end-policy
# after:
# - global:
#     apply:
#       - route_policy: A_NEW_ROUTE_POLICY
#     set:
#       community:
#         additive: true
#         community_name: (11011:1001)
#       weight: 20000
#   name: SIMPLE_GLOBAL_ROUTE_POLICY
# - else_section:
#     global:
#       drop: true
#   if_section:
#     condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
#     pass: true
#   name: SIMPLE_CONDITION_ROUTE_POLICY
# - else_section:
#     else_section:
#       drop: true
#     if_section:
#       condition: destination in A_RANDOM_POLICY
#       pass: true
#       set:
#         community:
#           additive: true
#           community_name: (101010:1)
#   if_section:
#     condition: as-path in (ios-regex '_3117_', ios-regex '_600_')
#     drop: true
#   name: COMPLEX_ROUTE_POLICY
# - else_section:
#     global:
#       pass: true
#   if_section:
#     condition: community matches-any (9119:1001) or community matches-any (11100:1001)
#     drop: true
#   name: COMPLEX_CONDITION_ROUTE_POLICY

# After state:
# -------------
#
# viosxr#show running-config | include route-policy
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy COMPLEX_ROUTE_POLICY
# route-policy COMPLEX_CONDITION_ROUTE_POLICY
#
# viosxr#show running-config route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
#   set weight 20000
#   set local-preference 200
#   set community (11011:1001) additive
#   apply A_NEW_ROUTE_POLICY
# end-policy
# viosxr#show running-config route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
#   if destination in SIMPLE_CONDITION_ROUTE_POLICY then
#     pass
#   else
#     drop
#   endif
# end-policy
# viosxr#show running-config route-policy COMPLEX_ROUTE_POLICY
# route-policy COMPLEX_ROUTE_POLICY
#   if as-path in (ios-regex '_3117_', ios-regex '_600_') then
#     drop
#   else
#     if destination in A_RANDOM_POLICY then
#       pass
#       set community (101010:1) additive
#       set local-preference 200
#     else
#       drop
#     endif
#   endif
# end-policy
# viosxr#show running-config route-policy COMPLEX_CONDITION_ROUTE_POLICY
# route-policy COMPLEX_CONDITION_ROUTE_POLICY
#   if community matches-any (9119:1001) or community matches-any (11100:1001) then
#     drop
#   else
#     pass
#   endif
# end-policy

# Using replaced

# Before state:
# -------------
#
# viosxr#show running-config | include route-policy
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy COMPLEX_ROUTE_POLICY
# route-policy COMPLEX_CONDITION_ROUTE_POLICY
#
# viosxr#show running-config route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
#   set weight 20000
#   set local-preference 200
#   set community (11011:1001) additive
#   apply A_NEW_ROUTE_POLICY
# end-policy
# viosxr#show running-config route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
#   if destination in SIMPLE_CONDITION_ROUTE_POLICY then
#     pass
#   else
#     drop
#   endif
# end-policy
# viosxr#show running-config route-policy COMPLEX_ROUTE_POLICY
# route-policy COMPLEX_ROUTE_POLICY
#   if as-path in (ios-regex '_3117_', ios-regex '_600_') then
#     drop
#   else
#     if destination in A_RANDOM_POLICY then
#       pass
#       set community (101010:1) additive
#       set local-preference 200
#     else
#       drop
#     endif
#   endif
# end-policy
# viosxr#show running-config route-policy COMPLEX_CONDITION_ROUTE_POLICY
# route-policy COMPLEX_CONDITION_ROUTE_POLICY
#   if community matches-any (9119:1001) or community matches-any (11100:1001) then
#     drop
#   else
#     pass
#   endif
# end-policy

- name: Replace the route-policy configuration
  cisco.iosxr.iosxr_route_maps:
    state: replaced
    config:
      - global:
          apply:
            - route_policy: A_NEW_ROUTE_POLICY
          set:
            community:
              additive: true
              community_name: (11011:1001)
            weight: 20000
        name: SIMPLE_GLOBAL_ROUTE_POLICY
      - else_section:
          global:
            drop: true
        if_section:
          condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
          pass: true
        name: VERY_SIMPLE_CONDITION_ROUTE_POLICY

# Task Output
# -----------
#
# before:
# - global:
#     apply:
#       - route_policy: A_NEW_ROUTE_POLICY
#     set:
#       community:
#         additive: true
#         community_name: (11011:1001)
#       weight: 20000
#   name: SIMPLE_GLOBAL_ROUTE_POLICY
# - else_section:
#     global:
#       drop: true
#   if_section:
#     condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
#     pass: true
#   name: SIMPLE_CONDITION_ROUTE_POLICY
# - else_section:
#     else_section:
#       drop: true
#     if_section:
#       condition: destination in A_RANDOM_POLICY
#       pass: true
#       set:
#         community:
#           additive: true
#           community_name: (101010:1)
#   if_section:
#     condition: as-path in (ios-regex '_3117_', ios-regex '_600_')
#     drop: true
#   name: COMPLEX_ROUTE_POLICY
# - else_section:
#     global:
#       pass: true
#   if_section:
#     condition: community matches-any (9119:1001) or community matches-any (11100:1001)
#     drop: true
#   name: COMPLEX_CONDITION_ROUTE_POLICY
# commands:
# - route-policy VERY_SIMPLE_CONDITION_ROUTE_POLICY
# - if destination in SIMPLE_CONDITION_ROUTE_POLICY then
# - pass
# - else
# - drop
# - endif
# - end-policy
# after:
# - global:
#     apply:
#       - route_policy: A_NEW_ROUTE_POLICY
#     set:
#       community:
#         additive: true
#         community_name: (11011:1001)
#       weight: 20000
#   name: SIMPLE_GLOBAL_ROUTE_POLICY
# - else_section:
#     global:
#       drop: true
#   if_section:
#     condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
#     pass: true
#   name: SIMPLE_CONDITION_ROUTE_POLICY
# - else_section:
#     else_section:
#       drop: true
#     if_section:
#       condition: destination in A_RANDOM_POLICY
#       pass: true
#       set:
#         community:
#           additive: true
#           community_name: (101010:1)
#   if_section:
#     condition: as-path in (ios-regex '_3117_', ios-regex '_600_')
#     drop: true
#   name: COMPLEX_ROUTE_POLICY
# - else_section:
#     global:
#       pass: true
#   if_section:
#     condition: community matches-any (9119:1001) or community matches-any (11100:1001)
#     drop: true
#   name: COMPLEX_CONDITION_ROUTE_POLICY
# - else_section:
#     global:
#       drop: true
#   if_section:
#     condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
#     pass: true
#   name: VERY_SIMPLE_CONDITION_ROUTE_POLICY

# After state:
# -------------
#
# viosxr#show running-config | include route-policy
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy COMPLEX_ROUTE_POLICY
# route-policy COMPLEX_CONDITION_ROUTE_POLICY
# route-policy VERY_SIMPLE_CONDITION_ROUTE_POLICY
#
# viosxr#show running-config route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
#   set weight 20000
#   set local-preference 200
#   set community (11011:1001) additive
#   apply A_NEW_ROUTE_POLICY
# end-policy
# viosxr#show running-config route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
#   if destination in SIMPLE_CONDITION_ROUTE_POLICY then
#     pass
#   else
#     drop
#   endif
# end-policy
# viosxr#show running-config route-policy COMPLEX_ROUTE_POLICY
# route-policy COMPLEX_ROUTE_POLICY
#   if as-path in (ios-regex '_3117_', ios-regex '_600_') then
#     drop
#   else
#     if destination in A_RANDOM_POLICY then
#       pass
#       set community (101010:1) additive
#       set local-preference 200
#     else
#       drop
#     endif
#   endif
# end-policy
# viosxr#show running-config route-policy COMPLEX_CONDITION_ROUTE_POLICY
# route-policy COMPLEX_CONDITION_ROUTE_POLICY
#   if community matches-any (9119:1001) or community matches-any (11100:1001) then
#     drop
#   else
#     pass
#   endif
# end-policy
# viosxr#show running-config route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
#   if destination in SIMPLE_CONDITION_ROUTE_POLICY then
#     pass
#   else
#     drop
#   endif
# end-policy
# viosxr#show running-config route-policy VERY_SIMPLE_CONDITION_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
#   if destination in SIMPLE_CONDITION_ROUTE_POLICY then
#     pass
#   else
#     drop
#   endif
# end-policy

# Using overridden

# Before state:
# -------------
#
# viosxr#show running-config | include route-policy
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy COMPLEX_ROUTE_POLICY
# route-policy COMPLEX_CONDITION_ROUTE_POLICY
#
# viosxr#show running-config route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
#   set weight 20000
#   set local-preference 200
#   set community (11011:1001) additive
#   apply A_NEW_ROUTE_POLICY
# end-policy
# viosxr#show running-config route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
#   if destination in SIMPLE_CONDITION_ROUTE_POLICY then
#     pass
#   else
#     drop
#   endif
# end-policy
# viosxr#show running-config route-policy COMPLEX_ROUTE_POLICY
# route-policy COMPLEX_ROUTE_POLICY
#   if as-path in (ios-regex '_3117_', ios-regex '_600_') then
#     drop
#   else
#     if destination in A_RANDOM_POLICY then
#       pass
#       set community (101010:1) additive
#       set local-preference 200
#     else
#       drop
#     endif
#   endif
# end-policy
# viosxr#show running-config route-policy COMPLEX_CONDITION_ROUTE_POLICY
# route-policy COMPLEX_CONDITION_ROUTE_POLICY
#   if community matches-any (9119:1001) or community matches-any (11100:1001) then
#     drop
#   else
#     pass
#   endif
# end-policy

- name: Override the route-policy configuration
  cisco.iosxr.iosxr_route_maps:
    state: overridden
    config:
      - global:
          apply:
            - route_policy: A_NEW_ROUTE_POLICY
          set:
            community:
              additive: true
              community_name: (11011:1001)
            weight: 20000
        name: SIMPLE_GLOBAL_ROUTE_POLICY
      - else_section:
          global:
            drop: true
        if_section:
          condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
          pass: true
        name: VERY_SIMPLE_CONDITION_ROUTE_POLICY

# Task Output
# -----------
#
# before:
# - global:
#     apply:
#       - route_policy: A_NEW_ROUTE_POLICY
#     set:
#       community:
#         additive: true
#         community_name: (11011:1001)
#       weight: 20000
#   name: SIMPLE_GLOBAL_ROUTE_POLICY
# - else_section:
#     global:
#       drop: true
#   if_section:
#     condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
#     pass: true
#   name: SIMPLE_CONDITION_ROUTE_POLICY
# - else_section:
#     else_section:
#       drop: true
#     if_section:
#       condition: destination in A_RANDOM_POLICY
#       pass: true
#       set:
#         community:
#           additive: true
#           community_name: (101010:1)
#   if_section:
#     condition: as-path in (ios-regex '_3117_', ios-regex '_600_')
#     drop: true
#   name: COMPLEX_ROUTE_POLICY
# - else_section:
#     global:
#       pass: true
#   if_section:
#     condition: community matches-any (9119:1001) or community matches-any (11100:1001)
#     drop: true
#   name: COMPLEX_CONDITION_ROUTE_POLICY
# commands:
# - route-policy VERY_SIMPLE_CONDITION_ROUTE_POLICY
# - if destination in SIMPLE_CONDITION_ROUTE_POLICY then
# - pass
# - else
# - drop
# - endif
# - end-policy
# - no route-policy SIMPLE_CONDITION_ROUTE_POLICY
# - no route-policy COMPLEX_ROUTE_POLICY
# - no route-policy COMPLEX_CONDITION_ROUTE_POLICY
# after:
# - global:
#     apply:
#       - route_policy: A_NEW_ROUTE_POLICY
#     set:
#       community:
#         additive: true
#         community_name: (11011:1001)
#       weight: 20000
#   name: SIMPLE_GLOBAL_ROUTE_POLICY
# - else_section:
#     global:
#       drop: true
#   if_section:
#     condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
#     pass: true
#   name: VERY_SIMPLE_CONDITION_ROUTE_POLICY

# After state:
# -------------
#
# viosxr#show running-config | include route-policy
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy VERY_SIMPLE_CONDITION_ROUTE_POLICY
#
# viosxr#show running-config route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
#   set weight 20000
#   set local-preference 200
#   set community (11011:1001) additive
#   apply A_NEW_ROUTE_POLICY
# end-policy
# viosxr#show running-config route-policy VERY_SIMPLE_CONDITION_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
#   if destination in SIMPLE_CONDITION_ROUTE_POLICY then
#     pass
#   else
#     drop
#   endif
# end-policy

# Using purged

# Before state:
# -------------
#
# viosxr#show running-config | include route-policy
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy COMPLEX_ROUTE_POLICY
#
# viosxr#show running-config route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
#   set weight 20000
#   set local-preference 200
#   set community (11011:1001) additive
#   apply A_NEW_ROUTE_POLICY
# end-policy
# viosxr#show running-config route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
#   if destination in SIMPLE_CONDITION_ROUTE_POLICY then
#     pass
#   else
#     drop
#   endif
# end-policy
# viosxr#show running-config route-policy COMPLEX_ROUTE_POLICY
# route-policy COMPLEX_ROUTE_POLICY
#   if as-path in (ios-regex '_3117_', ios-regex '_600_') then
#     drop
#   else
#     if destination in A_RANDOM_POLICY then
#       pass
#       set community (101010:1) additive
#       set local-preference 200
#     else
#       drop
#     endif
#   endif
# end-policy

- name: Purge or remove route-policy configuration
  cisco.iosxr.iosxr_route_maps:
    state: purged
    config:
      - name: COMPLEX_ROUTE_POLICY_NO_EXIST
      - name: COMPLEX_ROUTE_POLICY

# Task Output
# -----------
#
# before:
# - global:
#     apply:
#       - route_policy: A_NEW_ROUTE_POLICY
#     set:
#       community:
#         additive: true
#         community_name: (11011:1001)
#       weight: 20000
#   name: SIMPLE_GLOBAL_ROUTE_POLICY
# - else_section:
#     global:
#       drop: true
#   if_section:
#     condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
#     pass: true
#   name: SIMPLE_CONDITION_ROUTE_POLICY
# - else_section:
#     else_section:
#       drop: true
#     if_section:
#       condition: destination in A_RANDOM_POLICY
#       pass: true
#       set:
#         community:
#           additive: true
#           community_name: (101010:1)
#   if_section:
#     condition: as-path in (ios-regex '_3117_', ios-regex '_600_')
#     drop: true
#   name: COMPLEX_ROUTE_POLICY
# commands:
# - no route-policy COMPLEX_CONDITION_ROUTE_POLICY
# after:
# - global:
#     apply:
#       - route_policy: A_NEW_ROUTE_POLICY
#     set:
#       community:
#         additive: true
#         community_name: (11011:1001)
#       weight: 20000
#   name: SIMPLE_GLOBAL_ROUTE_POLICY
# - else_section:
#     global:
#       drop: true
#   if_section:
#     condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
#     pass: true
#   name: SIMPLE_CONDITION_ROUTE_POLICY

# After state:
# -------------
#
# viosxr#show running-config | include route-policy
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
#
# viosxr#show running-config route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
#   set weight 20000
#   set local-preference 200
#   set community (11011:1001) additive
#   apply A_NEW_ROUTE_POLICY
# end-policy
# viosxr#show running-config route-policy SIMPLE_CONDITION_ROUTE_POLICY
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
#   if destination in SIMPLE_CONDITION_ROUTE_POLICY then
#     pass
#   else
#     drop
#   endif
# end-policy

# Using rendered

- name: Render route-policy configuration
  cisco.iosxr.iosxr_route_maps:
    state: rendered
    config:
      - global:
          apply:
            - route_policy: A_NEW_ROUTE_POLICY
          set:
            community:
              additive: true
              community_name: (11011:1001)
            weight: 20000
        name: SIMPLE_GLOBAL_ROUTE_POLICY
      - else_section:
          global:
            drop: true
        if_section:
          condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
          pass: true
        name: SIMPLE_CONDITION_ROUTE_POLICY
      - else_section:
          else_section:
            drop: true
          if_section:
            condition: destination in A_RANDOM_POLICY
            pass: true
            set:
              community:
                additive: true
                community_name: (101010:1)
        if_section:
          condition: as-path in (ios-regex '_3117_', ios-regex '_600_')
          drop: true
        name: COMPLEX_ROUTE_POLICY
      - else_section:
          global:
            pass: true
        if_section:
          condition: community matches-any (9119:1001) or community matches-any (11100:1001)
          drop: true
        name: COMPLEX_CONDITION_ROUTE_POLICY

# Task Output
# -----------
#
# rendered:
# - route-policy SIMPLE_GLOBAL_ROUTE_POLICY
# - apply A_NEW_ROUTE_POLICY
# - set community (11011:1001) additive
# - set weight 20000
# - end-policy
# - route-policy SIMPLE_CONDITION_ROUTE_POLICY
# - if destination in SIMPLE_CONDITION_ROUTE_POLICY then
# - pass
# - else
# - drop
# - endif
# - end-policy
# - route-policy COMPLEX_ROUTE_POLICY
# - if as-path in (ios-regex '_3117_', ios-regex '_600_') then
# - drop
# - else
# - if destination in A_RANDOM_POLICY then
# - pass
# - set community (101010:1) additive
# - else
# - drop
# - endif
# - endif
# - end-policy
# - route-policy COMPLEX_CONDITION_ROUTE_POLICY
# - if community matches-any (9119:1001) or community matches-any (11100:1001) then
# - drop
# - else
# - pass
# - endif
# - end-policy

# Using parsed

# File: parsed.cfg
# ----------------
#
# route-policy SIMPLE_GLOBAL_ROUTE_POLICY
#   set weight 20000
#   set local-preference 200
#   set community (11011:1001) additive
#   apply A_NEW_ROUTE_POLICY
# end-policy
# !
# route-policy SIMPLE_CONDITION_ROUTE_POLICY
#   if destination in SIMPLE_CONDITION_ROUTE_POLICY then
#     pass
#   else
#     drop
#   endif
# end-policy
# !
# route-policy COMPLEX_ROUTE_POLICY
#   if as-path in (ios-regex '_3117_', ios-regex '_600_') then
#     drop
#   else
#     if destination in A_RANDOM_POLICY then
#       pass
#       set community (101010:1) additive
#       set local-preference 200
#     else
#       drop
#     endif
#   endif
# end-policy
# !
# route-policy COMPLEX_CONDITION_ROUTE_POLICY
#   if community matches-any (9119:1001) or community matches-any (11100:1001) then
#     drop
#   else
#     pass
#   endif
# end-policy

- name: Parse the provided configuration
  cisco.iosxr.iosxr_route_maps:
    running_config: "{{ lookup('file', 'iosxr_route_maps_conf.cfg') }}"
    state: parsed

# Task Output
# -----------
#
# parsed:
# - global:
#     apply:
#     - route_policy: A_NEW_ROUTE_POLICY
#     set:
#       community:
#         additive: true
#         community_name: (11011:1001)
#       weight: 20000
#   name: SIMPLE_GLOBAL_ROUTE_POLICY
# - else_section:
#     global:
#       drop: true
#   if_section:
#     condition: destination in SIMPLE_CONDITION_ROUTE_POLICY
#     pass: true
#   name: SIMPLE_CONDITION_ROUTE_POLICY
# - else_section:
#     else_section:
#       drop: true
#     if_section:
#       condition: destination in A_RANDOM_POLICY
#       pass: true
#       set:
#         community:
#           additive: true
#           community_name: (101010:1)
#   if_section:
#     condition: as-path in (ios-regex '_3117_', ios-regex '_600_')
#     drop: true
#   name: COMPLEX_ROUTE_POLICY
# - else_section:
#     global:
#       pass: true
#   if_section:
#     condition: community matches-any (9119:1001) or community matches-any (11100:1001)
#     drop: true
#   name: COMPLEX_CONDITION_ROUTE_POLICY
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(purged) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(purged) or C(purged)
  type: list
  sample:
    - route-policy APPLY_TEST_ROUTE_POLICY_COMPLEX
    - if destination in DEFAULT then
    - pass
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - route-policy APPLY_TEST_ROUTE_POLICY_COMPLEX
    - if destination in DEFAULT then
    - pass
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.route_maps.route_maps import (
    Route_mapsArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.route_maps.route_maps import (
    Route_maps,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Route_mapsArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Route_maps(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Paul Aiton, < @paultaiton >
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_subscription_info

version_added: "1.2.0"

short_description: Get Azure Subscription facts

description:
    - Get facts for a specific subscription or all subscriptions.

options:
    id:
        description:
            - Limit results to a specific subscription by id.
            - Mutually exclusive with I(name).
        type: str
    name:
        description:
            - Limit results to a specific subscription by name.
            - Mutually exclusive with I(id).
        aliases:
            - subscription_name
        type: str
    all:
        description:
            - If true, will return all subscriptions.
            - If false will omit disabled subscriptions (default).
            - Option has no effect when searching by id or name, and will be silently ignored.
        type: bool
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key:value'.
            - Option has no effect when searching by id or name, and will be silently ignored.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Paul Aiton (@paultaiton)
'''

EXAMPLES = '''
- name: Get facts for one subscription by id
  azure_rm_subscription_info:
    id: 00000000-0000-0000-0000-000000000000

- name: Get facts for one subscription by name
  azure_rm_subscription_info:
    name: "my-subscription"

- name: Get facts for all subscriptions, including ones that are disabled.
  azure_rm_subscription_info:
    all: true

- name: Get facts for subscriptions containing tags provided.
  azure_rm_subscription_info:
    tags:
      - testing
      - foo:bar
'''

RETURN = '''
subscriptions:
    description:
        - List of subscription dicts.
    returned: always
    type: list
    contains:
        display_name:
            description: Subscription display name.
            returned: always
            type: str
            sample: my-subscription
        fqid:
            description: Subscription fully qualified id.
            returned: always
            type: str
            sample: "/subscriptions/00000000-0000-0000-0000-000000000000"
        subscription_id:
            description: Subscription guid.
            returned: always
            type: str
            sample: "00000000-0000-0000-0000-000000000000"
        state:
            description: Subscription state.
            returned: always
            type: str
            sample: "'Enabled' or 'Disabled'"
        tags:
            description: Tags assigned to resource group.
            returned: always
            type: dict
            sample: { "tag1": "value1", "tag2": "value2" }
        tenant_id:
            description: Subscription tenant id
            returned: always
            type: str
            sample: "00000000-0000-0000-0000-000000000000"
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


AZURE_OBJECT_CLASS = 'Subscription'


class AzureRMSubscriptionInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str', aliases=['subscription_name']),
            id=dict(type='str'),
            tags=dict(type='list', elements='str'),
            all=dict(type='bool')
        )

        self.results = dict(
            changed=False,
            subscriptions=[]
        )

        self.name = None
        self.id = None
        self.tags = None
        self.all = False

        mutually_exclusive = [['name', 'id']]

        super(AzureRMSubscriptionInfo, self).__init__(self.module_arg_spec,
                                                      supports_check_mode=True,
                                                      supports_tags=False,
                                                      mutually_exclusive=mutually_exclusive,
                                                      facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.id and self.name:
            self.fail("Parameter error: cannot search subscriptions by both name and id.")

        result = []

        if self.id:
            result = self.get_item()
        else:
            result = self.list_items()

        self.results['subscriptions'] = result
        return self.results

    def get_item(self):
        self.log('Get properties for {0}'.format(self.id))
        item = None
        result = []

        try:
            item = self.subscription_client.subscriptions.get(self.id)
        except ResourceNotFoundError:
            pass

        result = self.to_dict(item)

        return result

    def list_items(self):
        self.log('List all items')
        try:
            response = self.subscription_client.subscriptions.list()
        except Exception as exc:
            self.fail("Failed to list all items - {0}".format(str(exc)))

        results = []
        for item in response:
            # If the name matches, return result regardless of anything else.
            # If name is not defined and either state is Enabled or all is true, and tags match, return result.
            if self.name and self.name.lower() == item.display_name.lower():
                results.append(self.to_dict(item))
            elif not self.name and (self.all or item.state == "Enabled"):
                results.append(self.to_dict(item))

        return results

    def to_dict(self, subscription_object):
        if self.has_tags(subscription_object.tags, self.tags):
            return dict(
                display_name=subscription_object.display_name,
                fqid=subscription_object.id,
                state=subscription_object.state,
                subscription_id=subscription_object.subscription_id,
                tags=subscription_object.tags,
                tenant_id=subscription_object.tenant_id
            )
        else:
            return dict()


def main():
    AzureRMSubscriptionInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: assurance_issues_ignore_v1
short_description: Resource module for Assurance Issues Ignore V1
description:
- Manage operation create of the resource Assurance Issues Ignore V1.
- >
   Ignores the given list of issues. The response contains the list of issues which were successfully ignored as well
   as the issues which are failed to ignore. For detailed information about the usage of the API, please refer to the
   Open API specification document - https //github.com/cisco-en-programmability/catalyst-center-api-
   specs/blob/main/Assurance/CE_Cat_Center_Org-IssuesLifecycle-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  issueIds:
    description: Issue Ids.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Issues IgnoreTheGivenListOfIssuesV1
  description: Complete reference of the IgnoreTheGivenListOfIssuesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!ignore-the-given-list-of-issues
notes:
  - SDK Method used are
    issues.Issues.ignore_the_given_list_of_issues_v1,

  - Paths used are
    post /dna/intent/api/v1/assuranceIssues/ignore,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.assurance_issues_ignore_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: '{{my_headers | from_json}}'
    issueIds:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "successfulIssueIds": [
          "string"
        ],
        "failureIssueIds": [
          "string"
        ]
      },
      "version": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2020 Suyeb Ansari (@suyeb786)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_vmbackuppolicy
version_added: '1.1.0'
short_description: Create or Delete Azure VM Backup Policy
description:
    - Create or Delete Azure VM Backup Policy.
options:
    name:
        description:
            - Policy Name.
        required: true
        type: str
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    vault_name:
        description:
            - Recovery Service Vault Name.
        required: true
        type: str
    time:
        description:
            - Retention times of retention policy in UTC.
        required: false
        default: '12:00'
        type: str
    weekdays:
        description:
            - List of days of the week.
        required: false
        default: ['Monday']
        type: list
        elements: str
    weeks:
        description:
            - List of weeks of month.
        required: false
        default: ['First']
        type: list
        elements: str
    months:
        description:
            - List of months of year of yearly retention policy.
        required: false
        default: ['January']
        type: list
        elements: str
    count:
        description:
            - Count of duration types. Retention duration is obtained by the counting the duration type Count times.
        required: false
        default: 1
        type: int
    state:
        description:
            - Assert the state of the protection item.
            - Use C(present) for Creating Backup Policy.
            - Use C(absent) for Deleting Backup Policy.
        default: present
        type: str
        choices:
            - present
            - absent
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Suyeb Ansari (@suyeb786)
'''

EXAMPLES = '''
- name: Create VM Backup Policy
  azure_rm_backvmuppolicy:
     name: 'myBackupPolicy'
     vault_name: 'myVault'
     resource_group: 'myResourceGroup'
     time: '18:00'
     weekdays: ['Monday', 'Thursday', 'Friday']
     weeks: ['First', 'Fourth']
     months: ['February', 'November']
     count: 4
     state: present
- name: Delete VM Backup Policy
  azure_rm_backvmuppolicy:
     name: 'myBackupPolicy'
     vault_name: 'myVault'
     resource_group: 'myResourceGroup'
     state: absent
'''

RETURN = '''
response:
    description:
        - The response about the current state of the backup policy.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample:  "/subscriptions/xxxxxxx/resourceGroups/resourcegroup_name/ \
            providers/Microsoft.RecoveryServices/vaults/myVault/backupPolicies/myBackup"
        name:
            description:
                - Backup Policy Name.
            returned: always
            type: str
            sample:  "myBackup"
        properties:
            description:
                - The backup policy properties.
            returned: always
            type: dict
            sample: {
                    "backupManagementType": "AzureIaasVM",
                    "schedulePolicy": {
                      "schedulePolicyType": "SimpleSchedulePolicy",
                      "scheduleRunFrequency": "Weekly",
                      "scheduleRunDays": [
                        "Monday",
                        "Wednesday",
                        "Thursday"
                      ],
                      "scheduleRunTimes": [
                        "2018-01-24T10:00:00Z"
                      ],
                      "scheduleWeeklyFrequency": 0
                    },
                    "retentionPolicy": {
                      "retentionPolicyType": "LongTermRetentionPolicy",
                      "weeklySchedule": {
                        "daysOfTheWeek": [
                          "Monday",
                          "Wednesday",
                          "Thursday"
                        ],
                        "retentionTimes": [
                          "2018-01-24T10:00:00Z"
                        ],
                        "retentionDuration": {
                          "count": 1,
                          "durationType": "Weeks"
                        }
                      },
                      "monthlySchedule": {
                        "retentionScheduleFormatType": "Weekly",
                        "retentionScheduleWeekly": {
                          "daysOfTheWeek": [
                            "Wednesday",
                            "Thursday"
                          ],
                          "weeksOfTheMonth": [
                            "First",
                            "Third"
                          ]
                        },
                        "retentionTimes": [
                          "2018-01-24T10:00:00Z"
                        ],
                        "retentionDuration": {
                          "count": 2,
                          "durationType": "Months"
                        }
                      },
                      "yearlySchedule": {
                        "retentionScheduleFormatType": "Weekly",
                        "monthsOfYear": [
                          "February",
                          "November"
                        ],
                        "retentionScheduleWeekly": {
                          "daysOfTheWeek": [
                            "Monday",
                            "Thursday"
                          ],
                          "weeksOfTheMonth": [
                            "Fourth"
                          ]
                        },
                        "retentionTimes": [
                          "2018-01-24T10:00:00Z"
                        ],
                        "retentionDuration": {
                          "count": 4,
                          "durationType": "Years"
                        }
                      }
                    },
                    "timeZone": "Pacific Standard Time",
                    "protectedItemsCount": 0
                   }
        type:
            description:
                - Resource type.
            returned: always
            type: str
            sample:  "Microsoft.RecoveryServices/vaults/backupPolicies"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
import time
import json


class VMBackupPolicy(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            vault_name=dict(
                type='str',
                required=True
            ),
            time=dict(
                type='str',
                default='12:00'
            ),
            weekdays=dict(
                type='list',
                elements='str',
                default=['Monday']
            ),
            weeks=dict(
                type='list',
                elements='str',
                default=['First']
            ),
            months=dict(
                type='list',
                elements='str',
                default=['January']
            ),
            count=dict(
                type='int',
                default=1
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.name = None
        self.time = None
        self.state = None
        self.vault_name = None
        self.count = None
        self.weekdays = None
        self.weeks = None
        self.months = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.url = None
        self.status_code = [200, 201, 202, 204]

        self.body = {}
        self.query_parameters = {}
        self.query_parameters['api-version'] = '2019-05-13'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        super(VMBackupPolicy, self).__init__(derived_arg_spec=self.module_arg_spec,
                                             supports_check_mode=True,
                                             supports_tags=True
                                             )

    def get_url(self):
        return '/subscriptions/' \
               + self.subscription_id \
               + '/resourceGroups/' \
               + self.resource_group \
               + '/providers/Microsoft.RecoveryServices' \
               + '/vaults' + '/' \
               + self.vault_name + '/' \
               + "backupPolicies/" \
               + self.name

    def set_schedule_run_time(self):
        return time.strftime("%Y-%m-%d", time.gmtime()) + "T" + self.time + ":00Z"

    def get_body(self):
        self.log('backup attributes {0}'.format(self.body))
        self.time = self.set_schedule_run_time()
        schedule_policy = dict()
        schedule_policy['schedulePolicyType'] = 'SimpleSchedulePolicy'
        schedule_policy['scheduleRunFrequency'] = 'Weekly'
        schedule_policy['scheduleRunTimes'] = [self.time]
        schedule_policy['scheduleRunDays'] = self.weekdays

        weekly_schedule = dict()
        weekly_schedule['daysOfTheWeek'] = ['Monday']
        weekly_schedule['retentionTimes'] = [self.time]
        weekly_schedule['retentionDuration'] = dict()
        weekly_schedule['retentionDuration']['count'] = self.count
        weekly_schedule['retentionDuration']['durationType'] = 'Weeks'

        monthly_schedule = dict()
        monthly_schedule['retentionScheduleFormatType'] = 'Weekly'
        monthly_schedule['retentionScheduleWeekly'] = dict()
        monthly_schedule['retentionScheduleWeekly']['daysOfTheWeek'] = self.weekdays
        monthly_schedule['retentionScheduleWeekly']['weeksOfTheMonth'] = self.weeks
        monthly_schedule['retentionTimes'] = [self.time]
        monthly_schedule['retentionDuration'] = dict()
        monthly_schedule['retentionDuration']['count'] = self.count
        monthly_schedule['retentionDuration']['durationType'] = 'Months'

        yearly_schedule = dict()
        yearly_schedule['retentionScheduleFormatType'] = 'Weekly'
        yearly_schedule['monthsOfYear'] = self.months
        yearly_schedule['retentionScheduleWeekly'] = dict()
        yearly_schedule['retentionScheduleWeekly']['daysOfTheWeek'] = self.weekdays
        yearly_schedule['retentionScheduleWeekly']['weeksOfTheMonth'] = self.weeks
        yearly_schedule['retentionTimes'] = [self.time]
        yearly_schedule['retentionDuration'] = dict()
        yearly_schedule['retentionDuration']['count'] = self.count
        yearly_schedule['retentionDuration']['durationType'] = 'Years'

        body = dict()
        body['properties'] = dict()
        body['properties']['backupManagementType'] = 'AzureIaasVM'
        body['properties']['timeZone'] = 'Pacific Standard Time'
        body['properties']['schedulePolicy'] = schedule_policy
        body['properties']['retentionPolicy'] = dict()
        body['properties']['retentionPolicy']['retentionPolicyType'] = 'LongTermRetentionPolicy'
        body['properties']['retentionPolicy']['weeklySchedule'] = weekly_schedule
        body['properties']['retentionPolicy']['monthlySchedule'] = monthly_schedule
        body['properties']['retentionPolicy']['yearlySchedule'] = yearly_schedule
        return body

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        self.inflate_parameters(self.module_arg_spec, self.body, 0)

        self.url = self.get_url()
        self.body = self.get_body()
        old_response = None
        response = None

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        old_response = self.get_resource()

        changed = False
        if self.state == 'present':
            if old_response is False:
                response = self.create_vm_backup_policy()
                changed = True
            else:
                response = old_response
        if self.state == 'absent':
            changed = True
            response = self.delete_vm_backup_policy()
        self.results['response'] = response
        self.results['changed'] = changed

        return self.results

    def create_vm_backup_policy(self):
        # self.log('Creating VM Backup Policy {0}'.format(self.))
        try:
            response = self.mgmt_client.query(
                self.url,
                'PUT',
                self.query_parameters,
                self.header_parameters,
                self.body,
                self.status_code,
                600,
                30,
            )
        except Exception as e:
            self.log('Error in creating Backup Policy.')
            self.fail('Error in creating Backup Policy {0}'.format(str(e)))

        if hasattr(response, 'body'):
            response = json.loads(response.body())
        elif hasattr(response, 'context'):
            response = response.context['deserialized_data']
        else:
            self.fail("Create or Updating fail, no match message return, return info as {0}".format(response))

        return response

    def delete_vm_backup_policy(self):
        # self.log('Deleting Backup Policy {0}'.format(self.))
        try:
            response = self.mgmt_client.query(
                self.url,
                'DELETE',
                self.query_parameters,
                self.header_parameters,
                None,
                self.status_code,
                600,
                30,
            )
        except Exception as e:
            self.log('Error attempting to delete Azure Backup policy.')
            self.fail('Error attempting to delete Azure Backup policy: {0}'.format(str(e)))

    def get_resource(self):
        # self.log('Fetch Backup Policy Details {0}'.format(self.))
        found = False
        try:
            response = self.mgmt_client.query(
                self.url,
                'GET',
                self.query_parameters,
                self.header_parameters,
                None,
                self.status_code,
                600,
                30,
            )
            found = True
        except Exception as e:
            self.log('Backup policy does not exist.')
        if found is True:
            response = json.loads(response.body())
            return response
        else:
            return False


def main():
    VMBackupPolicy()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Eric Girard <@netgirard>
# Copyright: (c) 2024, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "certified",
}

DOCUMENTATION = r"""
---
module: aci_access_spine_switch_selector
short_description: Manage Fabric Access Policy Spine Switch Port Selectors (infra:SpineS)
description:
- Manage Fabric Access Policy Spine Switch Port Selectors on Cisco ACI fabrics.
options:
  spine_switch_profile:
    description:
    - The name of the Fabric access policy spine switch profile.
    type: str
    aliases: [ spine_switch_profile_name, switch_profile, switch_profile_name ]
  spine_switch_selector:
    description:
    -  The name of the Fabric access spine switch port selector.
    type: str
    aliases: [ name, spine_switch_selector_name, switch_selector, switch_selector_name, access_port_selector, access_port_selector_name  ]
  description:
    description:
    - The description for the spine switch port selector.
    type: str
  policy_group:
    description:
    - The name of the fabric access policy group to be associated with the spine switch port selector.
    type: str
    aliases: [ policy_group_name ]
  selector_type:
    description:
    - The host port selector type.
    - If using a port block to specify range of switches, the type must be set to C(range).
    type: str
    choices: [ all, range ]
    aliases: [ type ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The I(spine_switch_profile) must exist before using this module in your playbook.
  The M(cisco.aci.aci_access_spine_switch_profile) module can be used for this.
- If a I(policy_group) is used, it must exist before using this module in your playbook.
  The M(cisco.aci.aci_switch_policy_group) module can be used for this.
seealso:
- module: cisco.aci.aci_access_spine_switch_profile
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(infra:SpineS) and B(infra:RsAccNodePGrp).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Eric Girard (@netgirard)
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Add a switch policy spine profile selector (with policy group)
  cisco.aci.aci_access_spine_switch_selector:
    host: apic
    username: admin
    password: SomeSecretPassword
    spine_switch_profile: sw_name
    spine_switch_selector: spine_selector_name
    selector_type: range
    policy_group: somepolicygroupname
    state: present
  delegate_to: localhost

- name: Query a switch policy spine profile selector
  cisco.aci.aci_access_spine_switch_selector:
    host: apic
    username: admin
    password: SomeSecretPassword
    spine_switch_profile: sw_name
    spine_switch_selector: spine_selector_name
    selector_type: range
    state: query
  delegate_to: localhost

- name: Query all switch policy spine profile selectors
  cisco.aci.aci_access_spine_switch_selector:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost

- name: Remove a switch policy spine profile selector
  cisco.aci.aci_access_spine_switch_selector:
    host: apic
    username: admin
    password: SomeSecretPassword
    spine_switch_profile: sw_name
    spine_switch_selector: spine_selector_name
    selector_type: range
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import MATCH_ACCESS_POLICIES_SELECTOR_TYPE


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        spine_switch_profile=dict(type="str", aliases=["spine_switch_profile_name", "switch_profile", "switch_profile_name"]),
        spine_switch_selector=dict(
            type="str",
            aliases=[
                "name",
                "spine_switch_selector_name",
                "switch_selector",
                "switch_selector_name",
                "access_port_selector",
                "access_port_selector_name",
            ],
        ),  # Not required for querying all objects
        description=dict(type="str"),
        policy_group=dict(type="str", aliases=["policy_group_name"]),
        selector_type=dict(type="str", choices=list(MATCH_ACCESS_POLICIES_SELECTOR_TYPE.keys()), aliases=["type"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["spine_switch_profile", "spine_switch_selector", "selector_type"]],
            ["state", "present", ["spine_switch_profile", "spine_switch_selector", "selector_type"]],
        ],
    )

    spine_switch_profile = module.params.get("spine_switch_profile")
    spine_switch_selector = module.params.get("spine_switch_selector")
    description = module.params.get("description")
    policy_group = module.params.get("policy_group")
    selector_type = MATCH_ACCESS_POLICIES_SELECTOR_TYPE.get(module.params.get("selector_type"))
    state = module.params.get("state")

    child_configs = []
    if policy_group is not None:
        child_configs.append(dict(infraRsSpineAccNodePGrp=dict(attributes=dict(tDn="uni/infra/funcprof/spaccnodepgrp-{0}".format(policy_group)))))

    aci = ACIModule(module)

    aci.construct_url(
        root_class=dict(
            aci_class="infraInfra",
            aci_rn="infra",
        ),
        subclass_1=dict(
            aci_class="infraSpineP",
            aci_rn="spprof-{0}".format(spine_switch_profile),
            module_object=spine_switch_profile,
            target_filter={"name": spine_switch_profile},
        ),
        subclass_2=dict(
            aci_class="infraSpineS",
            aci_rn="spines-{0}-typ-{1}".format(spine_switch_selector, selector_type),
            module_object=spine_switch_selector,
            target_filter={"name": spine_switch_selector},
        ),
        child_classes=["infraRsSpineAccNodePGrp"],
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="infraSpineS",
            class_config=dict(
                descr=description,
                name=spine_switch_selector,
                type=selector_type,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="infraSpineS")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}


DOCUMENTATION = '''
---
module: job_cancel
author: "Wayne Witzel III (@wwitzel3)"
short_description: Cancel an Automation Platform Controller Job.
description:
    - Cancel Automation Platform Controller jobs. See
      U(https://www.ansible.com/tower) for an overview.
options:
    job_id:
      description:
        - ID of the job to cancel
      required: True
      type: int
    fail_if_not_running:
      description:
        - Fail loudly if the I(job_id) can not be canceled
      default: False
      type: bool
extends_documentation_fragment: awx.awx.auth
'''

EXAMPLES = '''
- name: Cancel job
  job_cancel:
    job_id: job.id
'''

RETURN = '''
id:
    description: job id requesting to cancel
    returned: success
    type: int
    sample: 94
'''


from ..module_utils.controller_api import ControllerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        job_id=dict(type='int', required=True),
        fail_if_not_running=dict(type='bool', default=False),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    job_id = module.params.get('job_id')
    fail_if_not_running = module.params.get('fail_if_not_running')

    # Attempt to look up the job based on the provided name
    job = module.get_one(
        'jobs',
        **{
            'data': {
                'id': job_id,
            }
        }
    )

    if job is None:
        module.fail_json(msg="Unable to find job with id {0}".format(job_id))

    cancel_page = module.get_endpoint(job['related']['cancel'])
    if 'json' not in cancel_page or 'can_cancel' not in cancel_page['json']:
        module.fail_json(msg="Failed to cancel job, got unexpected response from the controller", **{'response': cancel_page})

    if not cancel_page['json']['can_cancel']:
        if fail_if_not_running:
            module.fail_json(msg="Job is not running")
        else:
            module.exit_json(**{'changed': False})

    results = module.post_endpoint(job['related']['cancel'], **{'data': {}})

    if results['status_code'] != 202:
        module.fail_json(msg="Failed to cancel job, see response for details", **{'response': results})

    module.exit_json(**{'changed': True})


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: task_info
short_description: Information module for Task Info
description:
- This module represents an alias of the module task_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description:
    - StartTime query parameter. This is the epoch start time from which tasks need to be fetched.
    type: str
  endTime:
    description:
    - EndTime query parameter. This is the epoch end time upto which audit records need to be fetched.
    type: str
  data:
    description:
    - Data query parameter. Fetch tasks that contains this data.
    type: str
  errorCode:
    description:
    - ErrorCode query parameter. Fetch tasks that have this error code.
    type: str
  serviceType:
    description:
    - ServiceType query parameter. Fetch tasks with this service type.
    type: str
  username:
    description:
    - Username query parameter. Fetch tasks with this username.
    type: str
  progress:
    description:
    - Progress query parameter. Fetch tasks that contains this progress.
    type: str
  isError:
    description:
    - IsError query parameter. Fetch tasks ended as success or failure. Valid values true, false.
    type: str
  failureReason:
    description:
    - FailureReason query parameter. Fetch tasks that contains this failure reason.
    type: str
  parentId:
    description:
    - ParentId query parameter. Fetch tasks that have this parent Id.
    type: str
  offset:
    description:
    - Offset query parameter.
    type: int
  limit:
    description:
    - Limit query parameter.
    type: int
  sortBy:
    description:
    - SortBy query parameter. Sort results by this field.
    type: str
  order:
    description:
    - Order query parameter. Sort order - asc or dsc.
    type: str
  taskId:
    description:
    - TaskId path parameter. UUID of the Task.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Task GetTaskByIdV1
  description: Complete reference of the GetTaskByIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-task-by-id
- name: Cisco DNA Center documentation for Task GetTasksOperationalTasksV1
  description: Complete reference of the GetTasksOperationalTasksV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-tasks-operational-tasks
notes:
  - SDK Method used are
    task.Task.get_task_by_id_v1,
    task.Task.get_tasks_operational_tasks_v1,

  - Paths used are
    get /dna/intent/api/v1/task,
    get /dna/intent/api/v1/task/{taskId},
  - It should be noted that this module is an alias of task_v1_info

"""

EXAMPLES = r"""
- name: Get all Task Info
  cisco.dnac.task_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: string
    endTime: string
    data: string
    errorCode: string
    serviceType: string
    username: string
    progress: string
    isError: string
    failureReason: string
    parentId: string
    offset: 0
    limit: 0
    sortBy: string
    order: string
  register: result

- name: Get Task Info by id
  cisco.dnac.task_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    taskId: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "additionalStatusURL": "string",
        "data": "string",
        "endTime": 0,
        "errorCode": "string",
        "errorKey": "string",
        "failureReason": "string",
        "id": "string",
        "instanceTenantId": "string",
        "isError": true,
        "lastUpdate": 0,
        "operationIdList": [
          "string"
        ],
        "parentId": "string",
        "progress": "string",
        "rootId": "string",
        "serviceType": "string",
        "startTime": 0,
        "username": "string",
        "version": 0
      },
      "version": "string"
    }
"""
